#--------------------------------------------------------------------------
#     This file is part of OASA - a free chemical python library
#     Copyright (C) 2003 Beda Kosata <beda@zirael.org>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     Complete text of GNU GPL can be found in the file gpl.txt in the
#     main directory of the program

#--------------------------------------------------------------------------


from molecule import molecule
from atom import atom
from bond import bond

import re
from periodic_table import periodic_table
import coords_generator


class linear_formula( object):

  def __init__( self, text="", valency=0):
    if text:
      self.molecule = self.parse_text( text, valency=valency)



  def parse_text( self, text, valency=0):
    is_formula = re.compile("^([A-Z][a-z]?[0-9]*[+-]?)*$")
    form = text
    if not is_formula.match( form):
      return None
    chunks = re.split( "([A-Z][a-z]?[0-9]?[+-]?)", form)
    print chunks
    mol = molecule()
    last_atom = None
    for chunk in chunks:
      if chunk:
        as = self.chunk_to_atoms( chunk)
        for a in as:
          mol.add_vertex( a)
          if last_atom:
            mol.add_edge( last_atom, a)
        last_atom = self.get_last_free_atom( a)
    return mol

  def chunk_to_atoms( self, chunk):
    m = re.match( "([A-Z][a-z]?)([0-9])?([+-])?", chunk)    
    name = m.group( 1)
    number = m.group( 2) and int( m.group(2)) or 1
    sign = m.group( 3) and (int( m.group(3)+'1')) or 0
    print name, number, sign
    ret = []
    for i in range( number):
      ret.append( atom( symbol=name, charge=sign))
    return ret


  def get_last_free_atom( self, a):
    if a.get_free_valency() > 0:
      return a
    for at in a.get_neighbors():
      if at.get_free_valency() > 0:
        return at
    print 'end', a
    return a


a = linear_formula( 'CFHClC+H2', valency=1)
m = a.molecule
coords_generator.calculate_coords( m)
for a in m.atoms:
  print a.get_x(), a.get_y()

coords_generator.show_mol( m)
